/**
 * \addtogroup Lcd
 * @{
 */

/*****************************************************************************/
/**
 * \file Lcd.h
 * HD44780 LCD display controller routines for 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 01-03-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef LCD_H
#define LCD_H

#include <REG51.h>
#include <intrins.h>
#include "Delay.h"

/** 
 * Hardware LCD display port assignment.
 * Adjust \b LCD_PORT to match hardware.
 * \hideinitializer
 */
#define LCD_PORT P2

#define INST_REG 0         // LCD instruction register
#define DATA_REG 1         // LCD data register
#define LINE1_ADDR 0x00    // Start of line 1 in the DD-Ram
#define LINE2_ADDR 0x40    // Start of line 2 in the DD-Ram
#define CLEAR_DISPLAY 0x01 // LCD display clear command


/*****************************************************************************
 *
 *                         Global Variable Declarations
 *
 *****************************************************************************/
sbit    E    = LCD_PORT^4;       // LCD I/O enable          pin 6
sbit    RS   = LCD_PORT^5;       // LCD I/O register select pin 4
sbit    RW   = LCD_PORT^6;       // LCD I/O R/W select      pin 5
sbit    SPKR = LCD_PORT^7;       // Speaker output  


/*****************************************************************************
 *
 *                              Macro Definitions
 *
 *****************************************************************************/

/** 
 * Sets lcd cursor position.
 * \param address The address to move cursor to as an \b unsigned \b char.
 * \return -
 * \hideinitializer
 */
#define LCD_SET_ADDR(address)  (lcd_wr_reg(INST_REG, (address) | 0x80))  

/** 
 * Gets lcd cursor position.
 * \param - 
 * \return The address of the cursor as an \b unsigned \b char.
 * \hideinitializer
 */
#define LCD_GET_ADDR()    (lcd_rd_reg(INST_REG) & 0x7f)


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

// extern char putchar(char c);     // defined in stdio.h

/** 
 * Lcd module initialisation function.
 * Initialises hardware and the HD44780 LCD display controller for 4 bit
 * operation.
 * \param -
 * \return -
 */
extern void lcd_init(void);

/** 
 * Lcd module audible alert.
 * Generates a variable length tone suitable for keyboard feedback and 
 * ^g bell alerts.
 * \param count The number of on/ off cycles to emit as an \b unsigned \b char.
 * \return -
 */
extern void lcd_beep(unsigned char count);

#endif

/** @} */
